@@ -3,7 +3,7 @@ use strict;
 use warnings;
 use v5.10;
 
-our $VERSION = '0.02';
+our $VERSION = '0.03';
 
 use Carp;
 use File::Find;
@@ -11,14 +11,15 @@ use File::Spec;
 use Linux::Inotify2;
 
 use parent 'Exporter';
-our %EXPORT_TAGS = (print => [qw(WATCH_DIR FOUND_FILE DELETE_FILE)]);
+our %EXPORT_TAGS = (print => [qw(WATCH_DIR FOUND_FILE DELETE_FILE CATCH_ERROR)]);
 our @EXPORT = ('watch', @{$EXPORT_TAGS{'print'}});
 $EXPORT_TAGS{all} = \@EXPORT;
 
 use constant {
     WATCH_DIR   => 1,
     FOUND_FILE  => 2,
-    DELETE_FILE => 4
+    DELETE_FILE => 4,
+    CATCH_ERROR => 8,
 };
 
 # function interface
@@ -43,8 +44,11 @@ sub new {
         print    => 0+($args{print} || 0),
         filter   => $args{filter},
         scan     => $args{scan},
+        catch    => $args{catch},
     }, $class;
 
+    $self->{catch} //= sub { } if ($self->{print} & CATCH_ERROR);
+
     $self->watch_recursive( $path );
 
     $self;
@@ -105,16 +109,27 @@ sub watch_directory {
 sub _callback {
     my ($self, $path) = @_;
 
-    if ($self->{filter} && $path =~ $self->{filter}) {
+    if ($self->{filter} && $path !~ $self->{filter}) {
         return;
     }
 
     say $path if ($self->{print} & FOUND_FILE);
-    if ( $self->{callback}->( $path ) ) {
-        if ( $self->{delete} ) {
-            say $path if ($self->{print} & DELETE_FILE); 
-            unlink $path;
+    
+    my $delete;
+    if ($self->{catch}) {
+        $delete = eval { $self->{callback}->($path) };
+        if ($@) {
+            print "$path: $@" if $self->{print} & CATCH_ERROR;
+            $self->{catch}->($path, $@);
+            return;
         }
+    } else {
+        $delete = $self->{callback}->($path);
+    }
+
+    if ( $delete && $self->{delete} ) {
+        say $path if ($self->{print} & DELETE_FILE); 
+        unlink $path;
     }
 }
 
@@ -156,11 +171,18 @@ File::Hotfolder - recursive watch directory for new or modified files
 
     # object interface
     File::Hotfolder->new(
-        watch    => '/some/directory',
-        callback => sub { 
+        watch    => '/some/directory',  # which directory to watch
+        callback => sub {               # what to do with each new/modified file
             my $path = shift;
             ...
         },
+        delete   => 1,                  # delete each file if callback returns true
+        filter   => qr/\.json$/,        # only watch selected files
+        print    => WATCH_DIR,          # show which directories are watched
+        catch    => sub {               # catch callback errors
+            my ($path, $error) = @_;
+            ...
+        }
     )->loop;
 
     # function interface
@@ -172,7 +194,7 @@ File::Hotfolder - recursive watch directory for new or modified files
 =head1 DESCRIPTION
 
 This module uses L<Linux::Inotify2> to recursively watch a directory for new or
-modified files. A callback is called on each file with its absolute path.
+modified files. A callback is called on each file with its path.
 
 Deletions and new subdirectories are not reported but new subdirectories will
 be watched as well.
@@ -205,9 +227,14 @@ Filter filenames with regular expression before passing to callback.
 
 =item print
 
-Print to STDOUT each new directory (C<< print & WATCH_DIR >>), each file path
-before callback execution (C<< print & FOUND_FILE >>), and/or each deletion
-(C<< print & DELETE_FILE >>).
+Print to STDOUT each new directory (C<WATCH_DIR>), each file path before
+callback execution (C<FOUND_FILE>), and/or each deletion (C<DELETE_FILE>).
+Also use C<CATCH_ERROR> (implying C<catch>) to print callback errors.
+
+=item catch
+
+Error callback for failing callbacks. Disabled by default, so a dying callback
+will terminate the program.
 
 =item scan
 
